/*
 *  mappix-waypoints
 *
 *  Copyright (C) 2006,2007 Nils Faerber <nils.faerber@kernelconcepts.de>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "mappix.h"
#include "geo-tools.h"


static void
fill_liststore(GtkListStore *store, GList *waypoints)
{
GtkTreeIter iter;
GList *lp;
waypoint *wpdata;
gdouble lat, lon;

	if (waypoints == NULL)
		return;
	if (waypoints->data == NULL)
		return;

	lp = waypoints;
	do {
		wpdata = (waypoint *)lp->data;
		unit2latlon(wpdata->unitx, wpdata->unity, lat, lon);
		gtk_list_store_append(store, &iter);
		gtk_list_store_set (store, &iter, 0, wpdata->name, 1, lat, 2, lon, 3, lp, -1);
	} while ((lp = g_list_next(lp)));
}


static void
wp_tv_add_columns (GtkTreeView *treeview, mapwin *Mapper)
{
GtkCellRenderer *renderer;
GtkTreeViewColumn *column;
// GtkTreeModel *model;

	// model = gtk_tree_view_get_model(Mapper->track_treeview);
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Name", renderer, "text", 0, NULL);
	//gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column), GTK_TREE_VIEW_COLUMN_FIXED);
	//gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 50);
	gtk_tree_view_append_column (treeview, column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Latitude", renderer, "text", 1, NULL);
	// gtk_tree_view_column_set_sort_column_id (column, 1);
	gtk_tree_view_append_column (treeview, column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Longitude", renderer, "text", 2, NULL);
	// gtk_tree_view_column_set_sort_column_id (column, 1);
	gtk_tree_view_append_column (treeview, column);

	gtk_tree_view_set_search_column (treeview, 0);
}


static void
delete_waypoint (GtkTreeView *treeview, GtkListStore *store, GtkWidget *parent, gpointer user_data)
{
mapwin *Mapper = (mapwin *)user_data;
waypoint *wpdata;
GtkTreeSelection *tree_selection;
GtkTreeIter iter;
GList *lp;
gint res;
GtkWidget *dialog;

	tree_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	if (!gtk_tree_selection_get_selected(tree_selection, NULL, &iter))
		return;
	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 3, &lp, -1);
	wpdata = lp->data;

	dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(parent)),
		GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_QUESTION,
		GTK_BUTTONS_YES_NO,
		"Are you sure you want to delete waypoint\n%s", wpdata->name);
	res = gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_hide (dialog);
	gtk_widget_destroy (dialog);
	if (res != GTK_RESPONSE_YES) {
		return;
	}
	// fprintf(stderr, "would delete %s\n", wpdata->name);
	gtk_list_store_remove(GTK_LIST_STORE(store), &iter);

	Mapper->waypoints = g_list_remove(lp, wpdata);
	g_free(wpdata);
}


static void
edit_waypoint (GtkTreeView *treeview, GtkListStore *store, GtkWidget *parent, gpointer user_data)
{
// mapwin *Mapper = (mapwin *)user_data;
waypoint *wpdata;
GtkTreeSelection *tree_selection;
GtkTreeIter iter;
GList *lp;
gint result;
GtkWidget *dialog, *hbox, *stock, *table, *label;
GtkWidget *entry_name, *entry_lat, *entry_lon;
gdouble lat, lon;
const gchar *tmp_name;

	tree_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	if (!gtk_tree_selection_get_selected(tree_selection, NULL, &iter))
		return;
	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 3, &lp, -1);
	wpdata = lp->data;
	unit2latlon(wpdata->unitx, wpdata->unity, lat, lon);

	dialog = gtk_dialog_new_with_buttons("Edit waypoint", GTK_WINDOW(gtk_widget_get_toplevel(parent)),
			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_STOCK_OK,
			GTK_RESPONSE_ACCEPT,
			GTK_STOCK_CANCEL,
			GTK_RESPONSE_REJECT,
			NULL);

	hbox = gtk_hbox_new (FALSE, 8);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 8);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 0);

	stock = gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG);
	gtk_box_pack_start (GTK_BOX (hbox), stock, FALSE, FALSE, 0);

	table = gtk_table_new (3, 4, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
	gtk_table_set_col_spacings (GTK_TABLE (table), 4);
	gtk_table_set_homogeneous(GTK_TABLE (table), FALSE);
	gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);

	label = gtk_label_new ("Name");
	gtk_table_attach_defaults (GTK_TABLE (table),
			label,
			0, 1, 0, 1);
	// track_combo = gtk_combo_box_new_text();
	entry_name = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(entry_name), wpdata->name);
	gtk_entry_set_editable(GTK_ENTRY(entry_name), TRUE);
	gtk_table_attach_defaults (GTK_TABLE (table), entry_name, 1, 2, 0, 1);

	label = gtk_label_new ("Latitude");
	gtk_table_attach_defaults (GTK_TABLE (table),
			label,
			0, 1, 1, 2);
	entry_lat = gtk_spin_button_new_with_range(0.0, 360.0, 0.00000001);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(entry_lat), 8);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(entry_lat), lat);
	gtk_table_attach_defaults (GTK_TABLE (table), entry_lat, 1, 2, 1, 2);

	label = gtk_label_new ("Longitude");
	gtk_table_attach_defaults (GTK_TABLE (table),
			label,
			0, 1, 2, 3);
	entry_lon = gtk_spin_button_new_with_range(0.0, 360.0, 0.00000001);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(entry_lon), 8);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(entry_lon), lon);
	gtk_table_attach_defaults (GTK_TABLE (table), entry_lon, 1, 2, 2, 3);

	gtk_widget_show_all (hbox);

	result = gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_hide (dialog);

	switch (result) {
		case GTK_RESPONSE_ACCEPT:
			tmp_name = gtk_entry_get_text(GTK_ENTRY(entry_name));
			g_free(wpdata->name);
			wpdata->name = g_strdup(tmp_name);
			lat = gtk_spin_button_get_value(GTK_SPIN_BUTTON(entry_lat));
			lon = gtk_spin_button_get_value(GTK_SPIN_BUTTON(entry_lon));
			latlon2unit(lat, lon, wpdata->unitx, wpdata->unity)
			gtk_list_store_set(GTK_LIST_STORE(store), &iter, 0, wpdata->name, 1, lat, 2, lon, -1);
			break;
		default:
			break;
	}
	gtk_widget_destroy (dialog);
}


static void
new_waypoint (GtkTreeView *treeview, GtkListStore *store, GtkWidget *parent, gpointer user_data)
{
mapwin *Mapper = (mapwin *)user_data;
waypoint *wpdata;
GtkTreeIter iter;
gint result;
GtkWidget *dialog, *hbox, *stock, *table, *label;
GtkWidget *entry_name, *entry_lat, *entry_lon;
gdouble lat, lon;
const gchar *tmp_name;

	dialog = gtk_dialog_new_with_buttons("Edit waypoint", GTK_WINDOW(gtk_widget_get_toplevel(parent)),
			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_STOCK_OK,
			GTK_RESPONSE_ACCEPT,
			GTK_STOCK_CANCEL,
			GTK_RESPONSE_REJECT,
			NULL);

	hbox = gtk_hbox_new (FALSE, 8);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 8);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 0);

	stock = gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG);
	gtk_box_pack_start (GTK_BOX (hbox), stock, FALSE, FALSE, 0);

	table = gtk_table_new (3, 4, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
	gtk_table_set_col_spacings (GTK_TABLE (table), 4);
	gtk_table_set_homogeneous(GTK_TABLE (table), FALSE);
	gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);

	unit2latlon(Mapper->xmin, Mapper->ymin, lat, lon);

	label = gtk_label_new ("Name");
	gtk_table_attach_defaults (GTK_TABLE (table),
			label,
			0, 1, 0, 1);
	// track_combo = gtk_combo_box_new_text();
	entry_name = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(entry_name), "Label");
	gtk_entry_set_editable(GTK_ENTRY(entry_name), TRUE);
	gtk_table_attach_defaults (GTK_TABLE (table), entry_name, 1, 2, 0, 1);

	label = gtk_label_new ("Latitude");
	gtk_table_attach_defaults (GTK_TABLE (table),
			label,
			0, 1, 1, 2);
	entry_lat = gtk_spin_button_new_with_range(0.0, 360.0, 0.00000001);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(entry_lat), 8);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(entry_lat), lat);
	gtk_table_attach_defaults (GTK_TABLE (table), entry_lat, 1, 2, 1, 2);

	label = gtk_label_new ("Longitude");
	gtk_table_attach_defaults (GTK_TABLE (table),
			label,
			0, 1, 2, 3);
	entry_lon = gtk_spin_button_new_with_range(0.0, 360.0, 0.00000001);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(entry_lon), 8);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(entry_lon), lon);
	gtk_table_attach_defaults (GTK_TABLE (table), entry_lon, 1, 2, 2, 3);

	gtk_widget_show_all (hbox);

	result = gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_hide (dialog);

	switch (result) {
		case GTK_RESPONSE_ACCEPT:
			tmp_name = gtk_entry_get_text(GTK_ENTRY(entry_name));
			wpdata = (waypoint *)g_malloc(sizeof(waypoint));
			wpdata->name = g_strdup(tmp_name);
			lat = gtk_spin_button_get_value(GTK_SPIN_BUTTON(entry_lat));
			lon = gtk_spin_button_get_value(GTK_SPIN_BUTTON(entry_lon));
			latlon2unit(lat, lon, wpdata->unitx, wpdata->unity)
			Mapper->waypoints = g_list_prepend(Mapper->waypoints, wpdata);
			gtk_list_store_append(store, &iter);
			gtk_list_store_set(GTK_LIST_STORE(store), &iter, 0, wpdata->name, 1, lat, 2, lon, -1);
			break;
		default:
			break;
	}
	gtk_widget_destroy (dialog);
}


void
on_editview_waypoints (GtkWidget *widget, gpointer user_data)
{
mapwin *Mapper = (mapwin *)user_data;
GtkWidget *dialog;
gint dresult;
GtkWidget *w;
GtkWidget *wp_treeview;
GtkTreeModel *tree_model;
gboolean changed=FALSE;

	dialog = gtk_dialog_new_with_buttons ("My dialog", GTK_WINDOW(gtk_widget_get_toplevel(Mapper->mainwin)),
			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			"Edit", 1,
			"Delete", 2,
			"New", 3,
			GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
			NULL);

	/* track list in treeview */
	w = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (w), GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (w), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), w, TRUE, TRUE, 0);

	gtk_widget_show(w);
	tree_model = GTK_TREE_MODEL(gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_POINTER));
	wp_treeview = gtk_tree_view_new_with_model (tree_model);
#if 0
#ifdef HILDON
	g_object_set(G_OBJECT(Mapper->track_treeview), "allow-checkbox-mode", FALSE, NULL);
	gtk_tree_view_set_hover_selection(GTK_TREE_VIEW(Mapper->track_treeview), FALSE);
#endif
#endif
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (wp_treeview), TRUE);
	g_object_unref (tree_model);
	gtk_container_add (GTK_CONTAINER (w), wp_treeview);
	gtk_widget_show(wp_treeview);
	wp_tv_add_columns(GTK_TREE_VIEW (wp_treeview), Mapper);

	fill_liststore(GTK_LIST_STORE(tree_model), Mapper->waypoints);

	gtk_widget_show_all (dialog);

	dresult = 0;
	while (dresult >= 0) {
		dresult = gtk_dialog_run(GTK_DIALOG(dialog));

		switch (dresult) {
			case 1:
				edit_waypoint(GTK_TREE_VIEW(wp_treeview), GTK_LIST_STORE(tree_model), dialog, Mapper);
				changed = TRUE;
				break;
			case 2:
				delete_waypoint(GTK_TREE_VIEW(wp_treeview), GTK_LIST_STORE(tree_model), dialog, Mapper);
				changed = TRUE;
				break;
			case 3:
				new_waypoint(GTK_TREE_VIEW(wp_treeview), GTK_LIST_STORE(tree_model), dialog, Mapper);
				changed = TRUE;
				break;
			default:
				break;
		}
	}

	gtk_widget_hide(dialog);

	gtk_widget_destroy(dialog);
	if (changed)
		do_redraw(Mapper);
}
